//===========================================================================
// PROJECT : StatusIcon
// DESC    : Show a icon on the taskbar's tray.
// VERSION : 1.03
//  Copyright (c) 1997 by Eddie Carpenter (ecarpenter@itex.co.za)
//
//   You are hereby granted a royalty-free unlimited distribution
//   license to compile the components in this source file into
//   your applications.
//
//   This source file may be freely distributed through online networks
//   so long as no modifications are made to this file and no fee is
//   charged to obtain this file, other than normal online connection
//   charges.
//
//   These components may NOT be distributed as source code or
//   compiled DCU on diskette, CDRom, or as part of a
//   product without prior written consent of the author.
//
//   All rights not explicitly granted here are reserved by the author.
//
//   For revision information see StatusIcon.cpp
//===========================================================================
#ifndef StatusIconH
#define StatusIconH

#include <vcl\vcl.h>
#include <shellapi.h>
#include <DsgnWnds.hpp>
#include <DsgnIntf.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Buttons.hpp>
#include "aboutdlg.h"

//------------------------------------------------------------------------
// TStatusIcon class definition
//------------------------------------------------------------------------
#define WM_TRAYICON WM_USER+10
class PACKAGE TStatusIcon : public TComponent
{
private:
   NOTIFYICONDATAA  nid;
   bool             DoubleClick;
   bool             FShowHint;
   bool             FEnabled;
   int              FIconIndex;
   TPopupMenu      *FPopupMenu;
   TIcon           *FIcon;
   TImageList      *FIcons;
   TMouseMoveEvent  FOnMouseMove;
   TNotifyEvent     FOnClick;
   TNotifyEvent     FOnDblClick;
   TMouseEvent      FOnMouseDown;
   TMouseEvent      FOnMouseUp;
   bool             FHideAppl;
   AnsiString       FHint;
   HANDLE           FHandle;
   TAboutProperty  *FAbout;

   //---------------------------------------------------------[ SetIcons ]---
   // DESC: Use by the SystrayIcons property to set the icon list
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall SetIcons(TImageList *Icons); // (I) Pointer to TImagelist

   //-----------------------------------------------------[ SetIconIndex ]---
   // DESC: Use by the IconIndex property to select a icon in SystrayIcons.
   //       If the Index is out of range it will be ignored.
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall SetIconIndex(int Index);  // (I) Index of icon in iconlist

   //------------------------------------------------------[ SetEnabled ]---
   // DESC: Use by the Enabled property to enable the icon.
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall SetEnabled(bool Value);

   //------------------------------------------------------[ SetShowHint ]---
   // DESC: Use by the ShowHint property to show the hint above the icon.
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall SetShowHint(bool Value);

   //-----------------------------------------------------[ SetPopupMenu ]---
   // DESC: Use by the PopupMenu property to assign the popupmenu
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall SetPopupMenu(TPopupMenu *Value);

   //------------------------------------------------------[ SetHideAppl ]---
   // DESC: Use by the HideApplication property to Hide or Show the Application
   //       on the task bar  
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall SetHideAppl(bool Value);

   //----------------------------------------------------------[ SetHint ]---
   // DESC: Use by the Hint property to update the icon with a new Hint
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall SetHint(AnsiString Value);

   //----------------------------------------------------[ WMLButtonDown ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMLButtonDown(TWMLButtonDown &Message); //(I) The Message

   //----------------------------------------------------[ WMRButtonDown ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMRButtonDown(TWMRButtonDown &Message); //(I) The Message

   //----------------------------------------------------[ WMMButtonDown ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMMButtonDown(TWMMButtonDown &Message); //(I) The Message

   //--------------------------------------------------[ WMLButtonDblClk ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMLButtonDblClk(TWMLButtonDblClk &Message); //(I) The Message

   //------------------------------------------------------[ WMMouseMove ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMMouseMove(TWMMouseMove &Message); //(I) The Message

   //------------------------------------------------------[ WMLButtonUp ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMLButtonUp(TWMLButtonUp &Message); //(I) The Message

   //------------------------------------------------------[ WMRButtonUp ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMRButtonUp(TWMRButtonUp &Message); //(I) The Message

   //------------------------------------------------------[ WMMButtonUp ]---
   // DESC: Handle the this windows event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void __fastcall WMMButtonUp(TWMMButtonUp &Message); //(I) The Message

   //------------------------------------------------------------[ Click ]---
   // DESC: Internaly use to generate a click event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void Click();

   //---------------------------------------------------------[ DblClick ]---
   // DESC: Internaly use to generate a double event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void DblClick();

   //--------------------------------------------------------[ MouseMove ]---
   // DESC: Internaly use to generate a mouse move event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void MouseMove(TShiftState Shift,int  X, int Y);

   //----------------------------------------------------------[ MouseUp ]---
   // DESC: Internaly use to generate a mouse up event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void MouseUp(TWMMouse Message,TMouseButton  Button);

   //--------------------------------------------------------[ MouseDown ]---
   // DESC: Internaly use to generate a mouse down event
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void MouseDown(TWMMouse Message, TMouseButton Button, TShiftState Shift);

   //-------------------------------------------------------[ UpdateIcon ]---
   // DESC: Update/Create/Remove the icon 
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
   void UpdateIcon();

protected:

   //----------------------------------------------------------[ WndProc ]---
   // DESC: Handle the windows message for the icon
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: none
   //------------------------------------------------------------------------
  virtual void __fastcall WndProc(Messages::TMessage &Message);

public:
   //------------------------------------------------------[ TStatusIcon ]---
   // DESC: This is the Constructor for the class
   // PRE:  None
   // POST: The class is constructed
   // RETURN: Pointer an instance of the class
   //------------------------------------------------------------------------
   __fastcall TStatusIcon(TComponent* Owner);

   //-----------------------------------------------------[ ~TStatusIcon ]---
   // DESC: This is the Destructor for the class
   // PRE:  The class was properly constructed
   // POST: The class is destructed
   // RETURN: None
   //------------------------------------------------------------------------
   __fastcall virtual ~TStatusIcon(void);

   //-----------------------------------------------------------[ Handle ]---
   // DESC: Handle to the window
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property HANDLE Handle = {read=FHandle};

__published:

   //--------------------------------------------------------[ IconIndex ]---
   // DESC: Set the Index of the icon. See SetIconIndex
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property TAboutProperty *About = {read=FAbout, write=FAbout};

   //--------------------------------------------------------[ IconIndex ]---
   // DESC: Set the Index of the icon. See SetIconIndex
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property int  IconIndex = {read=FIconIndex, write=SetIconIndex};

   //-----------------------------------------------------[ SystrayIcons ]---
   // DESC: Set/Get the SystrayIcons'. See SetIcons
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property TImageList *SystrayIcons = {read=FIcons, write=SetIcons};

   //---------------------------------------------------------[ ShowHint ]---
   // DESC: Set/Get ShowHint status
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property bool  ShowHint = {read=FShowHint, write=SetShowHint};

   //------------------------------------------------------[ OnMouseMove ]---
   // DESC: OnMousemove user callback.
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property TMouseMoveEvent OnMouseMove = {read=FOnMouseMove,write=FOnMouseMove};

   //----------------------------------------------------------[ OnClick ]---
   // DESC: OnClick user callback
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property TNotifyEvent OnClick = {read=FOnClick,write=FOnClick};

   //-------------------------------------------------------[ OnDblClick ]---
   // DESC: OnDblClick user callback
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
   __property TNotifyEvent OnDblClick = {read=FOnDblClick,write=FOnDblClick};

   //------------------------------------------------------[ OnMouseDown ]---
   // DESC: OnMouseDown user callback
   // PRE:  The class was properly constructed
   // POST: none
   // RETURN: PostboxResult
   //------------------------------------------------------------------------
  __property TMouseEvent  OnMouseDown = {read=FOnMouseDown,write=FOnMouseDown};

   //-------------------------------------------------------------[ OnMouseUp ]---
   // DESC: OnMouseUp user callback
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
  __property TMouseEvent OnMouseUp = {read=FOnMouseUp,write=FOnMouseUp};

   //----------------------------------------------------------[ Enabled ]---
   // DESC: Enabled or  disable this control
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
  __property bool Enabled = {read=FEnabled, write=SetEnabled};

   //-------------------------------------------------------------[ Hint ]---
   // DESC: Set the hint to be displayed
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
  __property AnsiString Hint = {read=FHint, write=SetHint};

   //--------------------------------------------------------[ PopupMenu ]---
   // DESC: Set/Get pointer to the PopupMenu
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
  __property TPopupMenu *PopupMenu = {read=FPopupMenu, write=SetPopupMenu};

   //--------------------------------------------------[ HideApplication ]---
   // DESC: Hide the Application from the Taskbar
   // PRE:  The class was properly constructed
   // POST: none
   //------------------------------------------------------------------------
  __property bool HideApplication = {read=FHideAppl, write=SetHideAppl};


 BEGIN_MESSAGE_MAP
   MESSAGE_HANDLER(WM_LBUTTONDOWN, TWMLButtonDown, WMLButtonDown);
   MESSAGE_HANDLER(WM_RBUTTONDOWN, TWMRButtonDown, WMRButtonDown);
   MESSAGE_HANDLER(WM_MBUTTONDOWN, TWMMButtonDown, WMMButtonDown);
   MESSAGE_HANDLER(WM_LBUTTONDBLCLK, TWMLButtonDblClk, WMLButtonDblClk);
   MESSAGE_HANDLER(WM_MOUSEMOVE, TWMMouseMove, WMMouseMove);
   MESSAGE_HANDLER(WM_LBUTTONUP, TWMLButtonUp, WMLButtonUp);
   MESSAGE_HANDLER(WM_RBUTTONUP, TWMRButtonUp, WMRButtonUp);
   MESSAGE_HANDLER(WM_MBUTTONUP, TWMMButtonUp, WMMButtonUp);
 END_MESSAGE_MAP(TComponent);

};
//---------------------------------------------------------------------------
//
//                               T H E   E N D
//
//---------------------------------------------------------------------------
#endif
